"""Utilities."""
from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING, Optional, Union

if TYPE_CHECKING:
    import click


def click_directory(
    ctx: Optional[click.Context],  # pylint: disable=unused-argument
    param: Optional[click.Parameter],  # pylint: disable=unused-argument
    value: Union[Path, str],
) -> Path:
    """Click callback to convert str to Path and ensure its a directory.

    Args:
        ctx: Click context.
        param: The click parameter.
        value: Value provided to click.

    """
    if isinstance(value, str):
        value = Path(value)
    if value.is_file():
        raise ValueError(f"{value} is a file; expected directory")
    value.mkdir(exist_ok=True, parents=True)
    return value



"""Mixins."""
from __future__ import annotations

import os
from functools import cached_property
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from pathlib import Path


class NestedFileMixin:
    """Mixin for a file that may be nested in directories for handling paths."""

    path: Path
    root: Path

    @cached_property
    def relative_path(self) -> str:
        """Relative location of a file in a directory.

        This can be used when moving a file between directories to ensure that
        their path relative to the new root directory remains the same as for
        the original root directory.

        """
        if self.path == self.root:
            return "./"
        return "./" + str(self.path.parent).replace(str(self.root), "").lstrip(os.sep)



"""Build SSM Documents from Dox in ``DOX_DIRECTORY``.

If ``DOX_DIRECTORY`` (absolute or relative) is omitted, ``./dox`` is used.

.. rubric:: Usage
.. code-block:: shell

  $ ssm-dox build [OPTIONS] [DOX_DIRECTORY]

.. rubric:: Options
.. code-block:: text

  -o, --output TEXT  path where built files should be placed
                     [default: ./ssm_documents]

.. rubric:: Example
.. code-block:: shell

  $ ssm-dox build
  $ ssm-dox build ./dox
  $ ssm-dox build ./dox --output ./ssm_documents

"""
from pathlib import Path

import click

from ...constants import DOCUMENTS_DIR, DOX_DIR
from ...finder import Finder
from .utils import click_directory


@click.command("build", short_help="build dox")
@click.argument("dox_directory", callback=click_directory, default=DOX_DIR)
@click.option(
    "-o",
    "--output",
    callback=click_directory,
    default=DOCUMENTS_DIR,
    help="path where built files should be placed",
    show_default=True,
)
def build(dox_directory: Path, output: Path) -> None:
    """Build SSM Documents from Dox in DOX_DIRECTORY.

    If DOX_DIRECTORY (absolute or relative) is omitted, ./dox is used.

    """
    finder = Finder(root_dir=dox_directory)
    for dox in finder.dox:
        dox.build(output)



"""SSM Document builder."""
import logging
from importlib.metadata import PackageNotFoundError, version

from ._logging import CustomLogger

logging.setLoggerClass(CustomLogger)

try:
    __version__ = version(__name__)
except PackageNotFoundError:
    # package not installed
    __version__ = "0.0.0"



"""SSM Documents."""
from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Optional, cast

from .exceptions import DocumentDoesNotExist
from .mixins import NestedFileMixin
from .models.document import SsmDocumentDataModel

if TYPE_CHECKING:
    from pathlib import Path

    from mypy_boto3_s3.client import S3Client

    from ._logging import CustomLogger

LOGGER = cast("CustomLogger", logging.getLogger(__name__))


class Document(NestedFileMixin):
    """SSM Document."""

    def __init__(
        self,
        *,
        content: Optional[SsmDocumentDataModel] = None,
        path: Path,
        root_dir: Path,
    ) -> None:
        """Instantiate class.

        Args:
            content: Contents of the Document.
            path: Path to the Document.
            root_dir: The root directory continaing the Document.

        """
        self._content = content
        self.name = path.name
        self.path = path
        self.root = root_dir

    @property
    def content(self) -> SsmDocumentDataModel:
        """Contents of the Document."""
        if not self._content:
            if self.path.is_file():
                self._content = SsmDocumentDataModel.parse_raw(self.path.read_bytes())
            else:
                raise DocumentDoesNotExist(self.path)
        return self._content

    @content.setter
    def content(self, value: SsmDocumentDataModel) -> None:
        """Set the value of contents.

        Args:
            value: New contents of the Document.

        """
        self._content = value

    def json(self, *, exclude_none: bool = True, indent: Optional[int] = 4) -> str:
        """Output contents as a JSON formatted string.

        Args:
            exclude_none: Exclude fields whose value is None.
            indent: Number of spaces per indent level.

        """
        return self.content.json(exclude_none=exclude_none, indent=indent)

    def publish(
        self, client: S3Client, *, bucket: str, prefix: Optional[str] = None
    ) -> None:
        """Publish document to S3.

        Args:
            client: S3 client.
            bucket: Name of the S3 Bucket.
            prefix: A prefix to append to the S3 Object key.

        """
        key = f"{self.relative_path.lstrip('./')}/{self.name}".lstrip("/")
        if prefix:
            key = f"{prefix.rstrip('/')}/{key}"
        client.put_object(
            ACL="private",
            Body=self.json().encode(),
            Bucket=bucket,
            ContentType="application/json",
            Key=key,
        )
        LOGGER.success("published %s to s3://%s/%s", self.name, bucket, key)

    def write(self, content: Optional[SsmDocumentDataModel] = None) -> Path:
        """Write contents to disk.

        Args:
            content: What to write to the file.

        """
        if content:
            self.content = content
        self.path.write_text(self.json() + "\n")  # insert new line at the end
        LOGGER.success("wrote to %s", self.path)
        return self.path



"""Custom logging."""
import logging
from enum import IntEnum
from typing import Any, Text, Union


class LogLevels(IntEnum):  # cov: ignore
    """All available log levels."""

    NOTSET = 0
    DEBUG = 10
    VERBOSE = 15
    INFO = 20
    NOTICE = 25
    WARNING = 30
    SUCCESS = 35
    ERROR = 40
    CRITICAL = 50

    @classmethod
    def has_value(cls, value: int) -> bool:
        """Check if IntEnum has a value."""
        return value in cls._value2member_map_  # pylint: disable=no-member


class CustomLogger(logging.Logger):  # cov: ignore
    """Extend built-in logger with additional levels."""

    def __init__(self, name: str, level: Union[int, Text] = logging.NOTSET) -> None:
        """Instantiate the class.

        Args:
            name: Logger name.
            level: Log level.

        """
        super().__init__(name, level)
        logging.addLevelName(LogLevels.VERBOSE, LogLevels.VERBOSE.name)
        logging.addLevelName(LogLevels.NOTICE, LogLevels.NOTICE.name)
        logging.addLevelName(LogLevels.SUCCESS, LogLevels.SUCCESS.name)

    def notice(self, msg: Union[Exception, str], *args: Any, **kwargs: Any) -> None:
        """Log 'msg % args' with severity `NOTICE`.

        Args:
            msg: String template or exception to use for the log record.

        """
        if self.isEnabledFor(LogLevels.NOTICE):
            self._log(LogLevels.NOTICE, msg, args, **kwargs)

    def success(self, msg: Union[Exception, str], *args: Any, **kwargs: Any) -> None:
        """Log 'msg % args' with severity `SUCCESS`.

        Args:
            msg: String template or exception to use for the log record.

        """
        if self.isEnabledFor(LogLevels.SUCCESS):
            self._log(LogLevels.SUCCESS, msg, args, **kwargs)

    def verbose(self, msg: Union[Exception, str], *args: Any, **kwargs: Any) -> None:
        """Log 'msg % args' with severity `VERBOSE`.

        Args:
            msg: String template or exception to use for the log record.

        """
        if self.isEnabledFor(LogLevels.VERBOSE):
            self._log(LogLevels.VERBOSE, msg, args, **kwargs)



"""Base classes for data models."""
from pydantic import BaseModel as _BaseModel


class BaseModel(_BaseModel):
    """Custom base class for data models."""

    def __bool__(self) -> bool:
        """Calculate the boolean value of the object."""
        return bool(
            self.dict(exclude_defaults=True, exclude_none=True, exclude_unset=True)
        )

    def __eq__(self, other: object) -> bool:
        """Calculate equality (==)."""
        if isinstance(other, BaseModel):
            return self.dict() == other.dict()
        if isinstance(other, str):
            return self.json(exclude_none=True, indent=4) == other
        return self.dict() == other

    def __ne__(self, other: object) -> bool:
        """Calculate inequality (!=)."""
        return not self == other



"""AWS SSM Document data model."""
from __future__ import annotations

from typing import Dict, List, Optional

from pydantic import validator

from .base import BaseModel
from .main_steps import AnyMainStep
from .parameter import SsmDocumentParameterDataModel


class SsmDocumentDataModel(BaseModel):
    """AWS SSM Document data model."""

    schemaVersion: str = "2.2"
    description: Optional[str] = None
    parameters: Optional[Dict[str, SsmDocumentParameterDataModel]] = None
    mainSteps: List[AnyMainStep]

    @validator("schemaVersion")
    @classmethod
    def _validate_supported_schema_version(cls, v: str) -> str:
        """Validate the schema version is supported."""
        split_version = tuple(int(i) for i in v.split("."))
        if split_version < (2, 2):
            raise ValueError("this tool only supports schemaVersion >= 2.2")
        return v



"""Explore a directory."""
from __future__ import annotations

import logging
import os
from functools import cached_property
from pathlib import Path
from typing import List

from .document import Document
from .dox import Dox

LOGGER = logging.getLogger(__name__)


class Finder:
    """Explore a directory."""

    def __init__(self, root_dir: Path) -> None:
        """Instantiate class.

        Args:
            root_dir: The root directory to explore.

        """
        self.root = root_dir

    @cached_property
    def documents(self) -> List[Document]:
        """List of documents found in the root directory."""
        result = [
            Document(path=f, root_dir=self.root) for f in self.root.rglob("*.json")
        ]
        LOGGER.info("found %s document(s)", len(result))
        return result

    @cached_property
    def dox(self) -> List[Dox]:
        """List of Dox found in the root directory."""
        result = [
            Dox(path=d, root_dir=self.root)
            for d in self.subdirectories
            if self.has_template(d)
        ]
        LOGGER.info("found %s dox", len(result))
        return result

    @cached_property
    def subdirectories(self) -> List[Path]:
        """List of subdirectories."""
        return self.scandir(self.root)

    @staticmethod
    def has_template(path: Path) -> bool:
        """Check if a directory contains a template file.

        Args:
            path: Path to check for a template file.

        """
        if list(path.glob("template.*")):
            return True
        return False

    @staticmethod
    def scandir(path: Path) -> List[Path]:
        """Scan directory to find all subdirectories.

        Args:
            path: Path to scan.

        """
        dirs: List[Path] = [Path(d.path) for d in os.scandir(path) if d.is_dir()]
        for d in list(dirs):
            dirs.extend(Finder.scandir(d))
        return dirs



"""AWS SSM Document mainStep base data model."""
from __future__ import annotations

from typing import Dict, List, Literal, Optional

from ..base import BaseModel


class SsmDocumentMainStepInputs(BaseModel):
    """AWS SSM Document mainStep.inputs base data model."""

    finallyStep: Optional[bool] = None
    onFailure: Optional[Literal["exit", "successAndExit"]] = None
    onSuccess: Optional[Literal["exit"]] = None


class SsmDocumentMainStep(BaseModel):
    """AWS SSM Document mainStep base data model."""

    action: str
    inputs: Optional[SsmDocumentMainStepInputs] = None
    name: str
    precondition: Optional[Dict[str, List[str]]] = None



"""Objects for working with Dox."""
from __future__ import annotations

import difflib
import logging
from functools import cached_property
from pathlib import Path
from typing import IO, TYPE_CHECKING, Any, Optional, cast

import yaml

from .document import Document
from .exceptions import DocumentDrift, TemplateNotFound
from .mixins import NestedFileMixin
from .models.document import SsmDocumentDataModel

if TYPE_CHECKING:
    from ._logging import CustomLogger

LOGGER = cast("CustomLogger", logging.getLogger(__name__))


class DoxLoader(yaml.SafeLoader):
    """Dox YAML loader."""

    def __init__(self, stream: IO[Any]) -> None:
        """Instantiate class."""
        try:
            self._root = Path(stream.name).parent
        except AttributeError:
            self._root = Path.cwd()
        super().__init__(stream)  # type: ignore

    def construct_include_script(self, node: yaml.Node) -> Any:
        """Handle !IncludeScript."""
        script = self._root / str(self.construct_scalar(node))  # type: ignore
        with open(script, "r") as f:
            return [line.rstrip() for line in f.readlines()]

    # pylint: disable=no-self-use,unused-argument
    def construct_linux_only(self, node: yaml.Node) -> Any:
        """Handle !LinuxOnly."""
        return {"StringEquals": ["platformType", "Linux"]}

    # pylint: disable=no-self-use,unused-argument
    def construct_windows_only(self, node: yaml.Node) -> Any:
        """Handle !WindowsOnly."""
        return {"StringEquals": ["platformType", "Windows"]}


DoxLoader.add_constructor("!IncludeScript", DoxLoader.construct_include_script)  # type: ignore
DoxLoader.add_constructor("!LinuxOnly", DoxLoader.construct_linux_only)  # type: ignore
DoxLoader.add_constructor("!WindowsOnly", DoxLoader.construct_windows_only)  # type: ignore


class Dox(NestedFileMixin):
    """Object representation of a raw SSM Document to be build."""

    def __init__(self, *, path: Path, root_dir: Path) -> None:
        """Instantiate class.

        Args:
            path: Path to the Dox directory.
            root_dir: The root directory continaing the Dox.

        """
        self.name = path.name
        self.path = path.absolute()
        self.root = root_dir.absolute()

    @cached_property
    def content(self) -> SsmDocumentDataModel:
        """Content of the Dox."""
        LOGGER.debug("loading %s...", self.template)
        with open(self.template, "r") as f:
            content = yaml.load(f, Loader=DoxLoader)  # type: ignore
        LOGGER.debug("parsing %s with data model...", self.template)
        return SsmDocumentDataModel.parse_obj(content)

    @cached_property
    def template(self) -> Path:
        """Template file."""
        extensions = ["yaml", "yml"]
        for ext in extensions:
            tmp_file = self.path / f"template.{ext}"
            if tmp_file.is_file():
                LOGGER.debug("found template file %s in %s", tmp_file.name, self.path)
                return tmp_file
        raise TemplateNotFound(self.path)

    def build(self, output_path: Path) -> Path:
        """Build Dox.

        Args:
            output_path: Path where built document will the saved.

        """
        document = self.get_document(output_path)
        document.path.parent.mkdir(exist_ok=True, parents=True)
        LOGGER.warning("building %s...", self.name)
        document.write(content=self.content)
        LOGGER.success("output %s to %s", self.name, document.path)
        return document.path

    def check(self, output_path: Path) -> None:
        """Check Dox against corresponding built document in output path.

        Args:
            output_path: Path where built documents are stored.

        """
        document = self.get_document(output_path)
        if self.content != document.content:
            raise DocumentDrift(
                document_content=document.content,
                document_path=document.path,
                dox_content=self.content,
                dox_path=self.template.parent,
            )
        LOGGER.success("%s is up to date", document.path)

    def diff(self, output_path: Path) -> None:
        """Diff Dox and corresponding built document in output path.

        Args:
            output_path: Path where built documents are stored.

        """
        dox_content = self.json().split("\n")
        doc_content = self.get_document(output_path).json().split("\n")
        differ = difflib.Differ()
        print("\n".join(differ.compare(doc_content, dox_content)))

    def get_document(self, output_path: Path) -> Document:
        """Get the built document that corresponds with this Dox in output path.

        Args:
            output_path: Path where built documents are stored.

        """
        return Document(path=self.get_document_path(output_path), root_dir=output_path)

    def get_document_path(self, output_path: Path) -> Path:
        """Get the path to the built document that corresponds with this Dox.

        Args:
            output_path: Path where built documents are stored.

        """
        return output_path / self.relative_path / f"{self.name}.json"

    def json(self, *, exclude_none: bool = True, indent: Optional[int] = 4) -> str:
        """Output contents as a JSON formatted string.

        Args:
            exclude_none: Exclude fields whose value is None.
            indent: Number of spaces per indent level.

        """
        return self.content.json(exclude_none=exclude_none, indent=indent)



"""AWS SSM Document mainStep data models."""
from typing import Union

from .aws_runpowershellscript import AwsRunPowerShellScript
from .aws_runshellscript import AwsRunShellScript

AnyMainStep = Union[AwsRunPowerShellScript, AwsRunShellScript]

__all__ = ["AwsRunPowerShellScript", "AwsRunShellScript"]



"""Compare Dox in to SSM Documents to ensure the match.

If ``DOX_DIRECTORY`` (absolute or relative) is omitted, ``./dox`` is used.
If ``DOCUMENTS_DIRECTORY`` (absolute or relative) is omitted, ``./ssm_documents`` is used.

.. rubric:: Usage
.. code-block:: shell

  $ ssm-dox check [OPTIONS] [DOX_DIRECTORY] [DOCUMENTS_DIRECTORY]

.. rubric:: Example
.. code-block:: shell

  $ ssm-dox check
  $ ssm-dox check ./dox
  $ ssm-dox check ./dox ./ssm_documents

"""
import logging
from pathlib import Path

import click

from ...constants import DOCUMENTS_DIR, DOX_DIR
from ...exceptions import DocumentDrift
from ...finder import Finder
from .utils import click_directory

LOGGER = logging.getLogger(__name__)


@click.command("check", short_help="check dox")
@click.argument("dox_directory", callback=click_directory, default=DOX_DIR)
@click.argument("documents_directory", callback=click_directory, default=DOCUMENTS_DIR)
@click.pass_context
def check(ctx: click.Context, documents_directory: Path, dox_directory: Path) -> None:
    """Compare Dox in to SSM Documents to ensure the match.

    If DOX_DIRECTORY (absolute or relative) is omitted, ./dox is used.
    If DOCUMENTS_DIRECTORY (absolute or relative) is omitted, ./ssm_documents is used.

    """
    finder = Finder(root_dir=dox_directory)
    for dox in finder.dox:
        try:
            dox.check(documents_directory)
        except DocumentDrift as err:
            LOGGER.error(err)
            dox.diff(documents_directory)
            ctx.exit(1)
    LOGGER.info("all documents are up to date")



"""AWS SSM Document mainStep aws:runPowerShellScript data model.

https://docs.aws.amazon.com/systems-manager/latest/userguide/ssm-plugins.html#aws-runPowerShellScript

"""
from __future__ import annotations

from typing import List, Literal, Optional

from .base import SsmDocumentMainStep, SsmDocumentMainStepInputs


class AwsRunPowerShellScriptInputs(SsmDocumentMainStepInputs):
    """AWS SSM Document mainStep aws:runPowerShellScript inputs data model."""

    runCommand: List[str] = []
    timeoutSeconds: Optional[str] = None
    workingDirectory: Optional[str] = None


class AwsRunPowerShellScript(SsmDocumentMainStep):
    """AWS SSM Document mainStep aws:runPowerShellScript data model.

    https://docs.aws.amazon.com/systems-manager/latest/userguide/ssm-plugins.html#aws-runPowerShellScript

    """

    action: Literal["aws:runPowerShellScript"]
    inputs: AwsRunPowerShellScriptInputs = AwsRunPowerShellScriptInputs()



"""Publish SSM Documents from ``DOCUMENTS_DIRECTORY`` to an S3 ``BUCKET``.

If DOCUMENTS_DIRECTORY (absolute or relative) is omitted, ``./ssm_documents`` is used.

.. rubric:: Usage
.. code-block:: shell

  $ ssm-dox publish [OPTIONS] BUCKET [DOCUMENTS_DIRECTORY]

.. rubric:: Options
.. code-block:: text

  -p, --prefix TEXT  prefix to append to S3 Object key  [default: dev]
  --profile TEXT     AWS profile name
  --region TEXT      AWS region where the bucket is located
  -h, --help         Show this message and exit.

.. rubric:: Example
.. code-block:: shell

  $ ssm-dox publish example-bucket
  $ ssm-dox publish example-bucket ./ssm_documents --prefix latest
  $ ssm-dox publish example-bucket ./ssm_documents --region us-east-1

"""
import logging
from pathlib import Path
from typing import Optional

import boto3
import click

from ...constants import DOCUMENTS_DIR
from ...finder import Finder
from .utils import click_directory

LOGGER = logging.getLogger(__name__)


@click.command("publish", short_help="publish documents")
@click.argument("bucket", required=True)
@click.argument("documents_directory", callback=click_directory, default=DOCUMENTS_DIR)
@click.option(
    "-p",
    "--prefix",
    default="dev",
    help="prefix to append to S3 Object key",
    show_default=True,
)
@click.option("--profile", default=None, help="AWS profile name")
@click.option("--region", default=None, help="AWS region where the bucket is located")
def publish(
    bucket: str,
    documents_directory: Path,
    *,
    prefix: Optional[str] = None,
    profile: Optional[str] = None,
    region: Optional[str] = None,
) -> None:
    """Publish SSM Documents from DOCUMENTS_DIRECTORY to an S3 ``BUCKET``.

    If DOCUMENTS_DIRECTORY (absolute or relative) is omitted, ./ssm_documents is used.

    """
    finder = Finder(root_dir=documents_directory)
    session = boto3.Session(profile_name=profile, region_name=region)  # type: ignore
    s3_client = session.client("s3")
    for doc in finder.documents:
        doc.publish(s3_client, bucket=bucket, prefix=prefix)



"""CLI logging setup."""
import logging
import os
from functools import cached_property
from typing import Any, Dict

import coloredlogs

LOGGER = logging.getLogger("ssm_dox")

LOG_FORMAT = "[%(programname)s] %(message)s"
LOG_FORMAT_VERBOSE = logging.BASIC_FORMAT
LOG_FIELD_STYLES = {
    "asctime": {},
    "hostname": {},
    "levelname": {},
    "message": {},
    "name": {},
    "prefix": {},
    "programname": {},
}
LOG_LEVEL_STYLES = {
    "critical": {"color": "red", "bold": True},
    "debug": {"color": "green"},
    "error": {"color": "red"},
    "info": {},
    "notice": {"color": "yellow"},
    "spam": {"color": "green", "faint": True},
    "success": {"color": "green", "bold": True},
    "verbose": {"color": "cyan"},
    "warning": {"color": 214},
}


class LogSettings:  # cov: ignore
    """CLI log settings."""

    ENV = {
        "field_styles": os.getenv("LOG_FIELD_STYLES"),
        "fmt": os.getenv("LOG_FORMAT"),
        "level_styles": os.getenv("LOG_LEVEL_STYLES"),
    }

    def __init__(
        self, *, debug: int = 0, no_color: bool = False, verbose: bool = False
    ):
        """Instantiate class.

        Args:
            debug: Debug level.
            no_color: Disable color in logs.
            verbose: Whether to display verbose logs.

        """
        self.debug = debug
        self.no_color = no_color
        self.verbose = verbose

    @property
    def coloredlogs(self) -> Dict[str, Any]:
        """Return settings for coloredlogs."""
        return {
            "fmt": self.fmt,
            "field_styles": self.field_styles,
            "level_styles": self.level_styles,
        }

    @cached_property
    def fmt(self) -> str:
        """Return log record format.

        If "LOG_FORMAT" exists in the environment, it will be used.

        """
        fmt = self.ENV["fmt"]
        if isinstance(fmt, str):
            return fmt
        if self.debug or self.no_color or self.verbose:
            return LOG_FORMAT_VERBOSE
        return LOG_FORMAT

    @cached_property
    def field_styles(self) -> Dict[str, Any]:
        """Return log field styles.

        If "LOG_FIELD_STYLES" exists in the environment, it will be
        used to update the LOG_FIELD_STYLES.

        """
        if self.no_color:
            return {}

        result = LOG_FIELD_STYLES.copy()
        if self.ENV["field_styles"]:
            result.update(
                coloredlogs.parse_encoded_styles(self.ENV["field_styles"])  # type: ignore
            )
        return result

    @cached_property
    def level_styles(self) -> Dict[str, Any]:
        """Return log level styles.

        If "LOG_LEVEL_STYLES" exists in the environment, it will be
        used to update the LOG_LEVEL_STYLES.

        """
        if self.no_color:
            return {}

        result = LOG_LEVEL_STYLES.copy()
        if self.ENV["level_styles"]:
            result.update(
                coloredlogs.parse_encoded_styles(self.ENV["level_styles"])  # type: ignore
            )
        return result

    @cached_property
    def log_level(self) -> int:
        """Return log level to use."""
        if self.debug:
            return logging.DEBUG
        if self.verbose:
            return logging.DEBUG
        return logging.INFO


def setup_logging(
    *, debug: int = 0, no_color: bool = False, verbose: bool = False
) -> None:
    """Configure log settings for CLI.

    Args:
        debug: Debug level (0-2).
        no_color: Whether to use colorized logs.
        verbose: Use verbose logging.

    """
    settings = LogSettings(debug=debug, no_color=no_color, verbose=verbose)

    coloredlogs.install(  # type: ignore
        settings.log_level, logger=LOGGER, **settings.coloredlogs
    )
    LOGGER.debug("runway log level: %s", LOGGER.getEffectiveLevel())

    if settings.debug == 2:  # cov: ignore
        coloredlogs.install(  # type: ignore
            settings.log_level,
            logger=logging.getLogger("botocore"),
            **settings.coloredlogs,
        )
        LOGGER.debug("set dependency log level to debug")
    LOGGER.debug("initalized logging")



"""Exceptions."""
from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from pathlib import Path

    from .models.document import SsmDocumentDataModel


class DocumentDoesNotExist(Exception):
    """The document trying to be used does not exist."""

    def __init__(self, path: Path) -> None:
        """Instantiate class.

        Args:
            path: Path that should be a Document but is not.

        """
        self.message = f"Document at path {path} does not exist"
        self.path = path
        super().__init__(self.message)


class DocumentDrift(Exception):
    """The currently loaded Dox does not match a corresponding built SSM Document."""

    def __init__(
        self,
        *,
        document_content: SsmDocumentDataModel,
        document_path: Path,
        dox_content: SsmDocumentDataModel,
        dox_path: Path,
    ) -> None:
        """Instantiate class.

        Args:
            document_content: Content of the built document.
            document_path: Path to the built document.
            dox_content: Content of the loaded Dox.
            dox_path: Path to the Dox directory.

        """
        self.document = document_content
        self.document_path = document_path
        self.dox = dox_content
        self.dox_path = dox_path
        self.message = (
            f"document at path {document_path} does not match Dox "
            f"loaded from {dox_path}"
        )
        super().__init__(self.message)


class TemplateNotFound(Exception):
    """Template not found in Dox directory."""

    def __init__(self, path: Path) -> None:
        """Instantiate class.

        Args:
            path: Path that should have a template but does not.

        """
        self.message = f"could not find a template file in {path}"
        self.path = path
        super().__init__(self.message)



"""AWS SSM Document parameter data model."""
from __future__ import annotations

from typing import Any, Dict, List, Literal, Optional, cast

from pydantic import validator

from .base import BaseModel

_ParameterType = Literal[
    "Boolean", "Integer", "MapList", "String", "StringList", "StringMap"
]


class SsmDocumentParameterDataModel(BaseModel):
    """AWS SSM Document parameter data model."""

    type: _ParameterType  # needs to be defined first for validators

    allowedPattern: Optional[str] = None
    allowedValues: Optional[List[str]] = None
    default: Any = None
    description: Optional[str] = None
    displayType: Optional[Literal["textarea", "testfield"]] = None
    maxChars: Optional[int] = None
    maxItems: Optional[int] = None
    minChars: Optional[int] = None
    minItems: Optional[int] = None

    @validator("default")
    @classmethod
    def _validate_default(cls, v: Any, values: Dict[str, Any]) -> Any:
        """Validate the schema version is supported."""
        if v is None:
            return v
        param_type = cast(_ParameterType, values.get("type"))
        if param_type == "Boolean":
            return bool(v)
        if param_type == "Integer":
            return int(v)
        if param_type == "MapList" and not isinstance(v, list):
            raise TypeError(f"{type(v)}; expected type List[Dict[str, str]]")
        if param_type == "String":
            return str(v)  # type: ignore
        if param_type == "StringList" and not isinstance(v, list):
            raise TypeError(f"{type(v)}; expected type List[str]")
        if param_type == "StringMap" and not isinstance(v, dict):
            raise TypeError(f"{type(v)}; expected type Dict[str, str]")  # type: ignore
        return v



"""SSM Document Builder CLI entrypoint."""
from __future__ import annotations

from typing import Any, Dict

import click

from .. import __version__
from . import commands
from .logs import setup_logging

CLICK_CONTEXT_SETTINGS: Dict[str, Any] = {
    "help_option_names": ["-h", "--help"],
    "max_content_width": 999,
}


@click.group(context_settings=CLICK_CONTEXT_SETTINGS)
@click.version_option(__version__, message="%(version)s")
def cli() -> None:
    """SSM Document Builder CLI."""
    setup_logging()


for cmd in commands.__all__:
    cli.add_command(getattr(commands, cmd))  # type: ignore



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ssm_dox',
 'ssm_dox._cli',
 'ssm_dox._cli.commands',
 'ssm_dox.models',
 'ssm_dox.models.main_steps']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=4.1,<5.3',
 'boto3>=1.17.23,<2.0.0',
 'click>=7.1.2,<8.0.0',
 'coloredlogs>=15.0,<16.0',
 'pydantic>=1.8.1,<2.0.0']

extras_require = \
{'docs': ['readthedocs-sphinx-search>=0.1.0,<0.2.0',
          'Sphinx>=3.5.2,<4.0.0',
          'sphinx-rtd-theme>=0.5.1,<0.6.0',
          'toml>=0.10.2,<0.11.0']}

entry_points = \
{'console_scripts': ['ssm-dox = ssm_dox._cli.main:cli']}

setup_kwargs = {
    'name': 'ssm-dox',
    'version': '0.1.1',
    'description': 'CLI tool for building and publishing SSM Documents.',
    'long_description': "# ssm-dox\n\n[![PyPi](https://img.shields.io/pypi/v/ssm-dox?style=flat)](https://pypi.org/project/ssm-dox/)\n[![code style: black](https://img.shields.io/badge/code%20style-black-000000.svg?style=flat)](https://github.com/psf/black)\n[![Documentation Status](https://readthedocs.org/projects/ssm-dox/badge/?version=latest)](https://ssm-dox.readthedocs.io/en/latest/?badge=latest)\n\nCLI tool for building and publishing SSM Documents.\n\n## What Is Dox?\n\n**Dox** is a directory containing the source code of an SSM document.\nNamed for it's corresponding SSM Document, the directory contains the following:\n\n- ``template.yaml`` or ``template.yml`` file **[REQUIRED]**\n- ``README.md`` file describing the SSM Document\n- external files to be included in the SSM Document (e.g. shell or PowerShell scripts)\n\n## Why Use Dox?\n\nUsing **Dox** to write an SSM Document allows for scripts and other items to be linted and tested outside of the SSM Document.\nThis enables CI/CD pipelines to easily validate the code before it gets deployed to AWS.\n",
    'author': 'Kyle Finley',
    'author_email': 'kyle@finley.sh',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ITProKyle/ssm-dox',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
